@@ -60,7 +60,7 @@ sub HexEncode($)
 }
 
 #------------------------------------------------------------------------------
-# Write profile to tEXt or zTXt chunk (zTXt if Zlib is available)
+# Write profile chunk (possibly compressed if Zlib is available)
 # Inputs: 0) outfile, 1) Raw profile type, 2) data ref
 #         3) profile header type (undef if not a text profile)
 # Returns: 1 on success
@@ -68,19 +68,29 @@ sub WriteProfile($$$;$)
 {
     my ($outfile, $rawType, $dataPt, $profile) = @_;
     my ($buff, $prefix, $chunk, $deflate);
-    if (eval { require Compress::Zlib }) {
+    if ($rawType ne $stdCase{exif} and eval { require Compress::Zlib }) {
         $deflate = Compress::Zlib::deflateInit();
     }
     if (not defined $profile) {
         # write ICC profile as compressed iCCP chunk if possible
-        return 0 unless $deflate;
-        $buff = $deflate->deflate($$dataPt);
-        return 0 unless defined $buff;
-        $buff .= $deflate->flush();
-        my %rawTypeChunk = ( icm => 'iCCP' );
-        $chunk = $rawTypeChunk{$rawType} or return 0;
-        $prefix = "$rawType\0\0";
-        $dataPt = \$buff;
+        if ($rawType eq 'icm') {
+            return 0 unless $deflate;
+            $chunk = 'iCCP';
+            $prefix = "$rawType\0\0";
+        } else {
+            $chunk = $rawType;
+            if ($rawType eq $stdCase{zxif}) {
+                $prefix = "\0" . pack('N', length $$dataPt); # (proposed compressed EXIF)
+            } else {
+                $prefix = '';   # standard EXIF
+            }
+        }
+        if ($deflate) {
+            $buff = $deflate->deflate($$dataPt);
+            return 0 unless defined $buff;
+            $buff .= $deflate->flush();
+            $dataPt = \$buff;
+        }
     } else {
         # write as ASCII-hex encoded profile in tEXt or zTXt chunk
         my $txtHdr = sprintf("\n$profile profile\n%8d\n", length($$dataPt));
@@ -106,7 +116,7 @@ sub WriteProfile($$$;$)
 }
 
 #------------------------------------------------------------------------------
-# Add iCCP to the PNG image if necessary (must come before PLTE and IDAT)
+# Add iCCP chunk to the PNG image if necessary (must come before PLTE and IDAT)
 # Inputs: 0) ExifTool object ref, 1) output file or scalar ref
 # Returns: true on success
 sub Add_iCCP($$)
@@ -120,7 +130,6 @@ sub Add_iCCP($$)
         if (defined $buff and length $buff and WriteProfile($outfile, 'icm', \$buff)) {
             $et->VPrint(0, "Created ICC profile\n");
             delete $$et{ADD_DIRS}{ICC_Profile}; # don't add it again
-            $$et{PNGDoneDir}{ICC_Profile} = 2;
         }
     }
     return 1;
@@ -134,28 +143,15 @@ sub Add_iCCP($$)
 sub DoneDir($$$;$)
 {
     my ($et, $dir, $outBuff, $nonStandard) = @_;
+    my $saveDir = $dir;
+    $dir = 'EXIF' if $dir eq 'IFD0';
     # don't add this directory again unless this is in a non-standard location
-    delete $$et{ADD_DIRS}{$dir} unless $nonStandard;
-    # handle problem with duplicate XMP when using PNGEarlyXMP option
-    return unless $dir eq 'XMP' and defined $$outBuff and length $$outBuff;
-    if ($nonStandard and $$et{DEL_GROUP}{$dir}) {
+    if (not $nonStandard) {
+        delete $$et{ADD_DIRS}{$dir};
+        delete $$et{ADD_DIRS}{IFD0} if $dir eq 'EXIF';
+    } elsif ($$et{DEL_GROUP}{$dir} or $$et{DEL_GROUP}{$saveDir}) {
         $et->VPrint(0,"  Deleting non-standard $dir\n");
         $$outBuff = '';
-    } elsif (not $$et{PNGDoneDir}{$dir}) {
-        $$et{PNGDoneDir}{$dir} = 1;   # set flag indicating the directory exists
-    } elsif ($$et{OPTIONS}{PNGEarlyXMP}) {
-        if ($$et{PNGDoneDir}{$dir} == 2) {
-            if ($$et{OPTIONS}{IgnoreMinorErrors}) {
-                $et->Warn("Deleted existing $dir");
-            } else {
-                $et->Error("Duplicate $dir created. Ignore to delete existing $dir", 1);
-                return;
-            }
-        } elsif ($et->Warn("Duplicate $dir. Ignore to delete", 2)) {
-            return; # warning not ignored: don't delete the duplicate
-        }
-        $et->VPrint(0,"  Deleting duplicate $dir\n");
-        $$outBuff = '';
     }
 }
 
@@ -227,15 +223,16 @@ sub BuildTextChunk($$$$$)
 #------------------------------------------------------------------------------
 # Add any outstanding new chunks to the PNG image
 # Inputs: 0) ExifTool object ref, 1) output file or scalar ref
-#         2-N) dirs to add (empty to add all, including PNG tags)
+#         2-N) dirs to add (empty to add all except EXIF 'IFD0', including PNG tags)
 # Returns: true on success
 sub AddChunks($$;@)
 {
     my ($et, $outfile, @add) = @_;
-    my ($addTags, $tag, $dir, $err, $tagTablePtr);
+    my ($addTags, $tag, $dir, $err, $tagTablePtr, $specified);
 
     if (@add) {
         $addTags = { }; # don't add any PNG tags
+        $specified = 1;
     } else {
         $addTags = $$et{ADD_PNG};    # add all PNG tags...
         delete $$et{ADD_PNG};        # ...once
@@ -263,7 +260,6 @@ sub AddChunks($$;@)
             my $cbuf = pack('N', CalculateCRC(\$data, undef));
             Write($outfile, $hdr, $data, $cbuf) or $err = 1;
             $et->VerboseValue("+ PNG:$$tagInfo{Name}", $val);
-            $$et{PNGDoneTag}{$tag} = 1;   # set flag indicating this tag was added
             ++$$et{CHANGED};
         }
     }
@@ -276,14 +272,22 @@ sub AddChunks($$;@)
             DirName => $dir,
         );
         if ($dir eq 'IFD0') {
-            $et->Warn('Creating non-standard EXIF in PNG', 1);
-            $et->VPrint(0, "Creating EXIF profile:\n");
+            next unless $specified;     # wait until specifically asked to write EXIF 'IFD0'
+            my $chunk = $stdCase{exif};
+            # (zxIf was not adopted)
+            #if ($et->Options('Compress')) {
+            #    if (eval { require Compress::Zlib }) {
+            #        $chunk = $stdCase{zxif};
+            #    } else {
+            #        $et->Warn("Creating uncompressed $stdCase{exif} chunk (Compress::Zlib not available)");
+            #    }
+            #}
+            $et->VPrint(0, "Creating $chunk chunk:\n");
             $$et{TIFF_TYPE} = 'APP1';
             $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::Exif::Main');
             $buff = $et->WriteDirectory(\%dirInfo, $tagTablePtr, \&Image::ExifTool::WriteTIFF);
             if (defined $buff and length $buff) {
-                $buff = $Image::ExifTool::exifAPP1hdr . $buff;
-                WriteProfile($outfile, 'APP1', \$buff, 'generic') or $err = 1;
+                WriteProfile($outfile, $chunk, \$buff) or $err = 1;
             }
         } elsif ($dir eq 'XMP') {
             $et->VPrint(0, "Creating XMP iTXt chunk:\n");
@@ -304,7 +308,7 @@ sub AddChunks($$;@)
                 Write($outfile, $hdr, $buff, $cbuf) or $err = 1;
             }
         } elsif ($dir eq 'IPTC') {
-            $et->Warn('Creating non-standard EXIF in PNG', 1);
+            $et->Warn('Creating non-standard IPTC in PNG', 1);
             $et->VPrint(0, "Creating IPTC profile:\n");
             # write new IPTC data (stored in a Photoshop directory)
             $dirInfo{DirName} = 'Photoshop';
@@ -323,7 +327,7 @@ sub AddChunks($$;@)
                 $et->Warn('Wrote ICC as a raw profile (no Compress::Zlib)');
             }
         } elsif ($dir eq 'PNG-pHYs') {
-            $et->VPrint(0, "Creating pHYs chunk:\n");
+            $et->VPrint(0, "Creating pHYs chunk (default 2834 pixels per meter):\n");
             $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::PNG::PhysicalPixel');
             my $blank = "\0\0\x0b\x12\0\0\x0b\x12\x01"; # 2834 pixels per meter (72 dpi)
             $dirInfo{DataPt} = \$blank;
@@ -338,8 +342,6 @@ sub AddChunks($$;@)
             next;
         }
         delete $$et{ADD_DIRS}{$dir};  # don't add again
-        # keep track of the directories that we added
-        $$et{PNGDoneDir}{$dir} = 2 if defined $buff and length $buff;
     }
     return not $err;
 }
@@ -377,7 +379,7 @@ strings).
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
